/* Copyright Jamie Honan, 2001.  Distributed under the GPL.
   This program comes with ABSOLUTELY NO WARRANTY.
   See the file COPYING for license details.
   */
#define KEYBD_EXTERN
#include "global.h"
#include "timer.h"
//#include "keysta.h"
#include "keymap.h"
#include "print.h"
#include "led.h"

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <stdio.h>
#include <avr/eeprom.h>
#include <avr/wdt.h>
#include <util/delay.h>     /* for _delay_ms() */

#include "usbdrv.h"


#define USB_LEVEL_SHIFT_PORT    PORTD
#define USB_LEVEL_SHIFT_DDR     DDRD

#define USB_LEVEL_SHIFT_PIN     6

#define INTERFACE_CHECK_TIME    5     // 50ms


int8_t usbmode = 1;

extern uint8_t usbmain(void);
extern uint8_t ps2main(void);




#ifdef DEBUG
void enable_printf(void)
{
	stdout = &mystdout;
    DDRD |= 0x01;

	UBRRH = (UBRR>>8);
	UBRRL = UBRR;
    
	UCSRA = 0x00;									   // asynchronous normal mode
	UCSRB = 0x08;									   // Tx enable, 8 data
	UCSRC = 0x86;									   // no parity, 1 stop, 8 data
}
#else
void enable_printf(void)
{
}
#endif





/* control zener diode for level shift signal line
    1 : TR on - 3v level
    0 : TR off - 5v level
*/
void DPpullEn(uint8_t enable)
{
    if (enable)
    {
        cbi(USB_LEVEL_SHIFT_DDR, USB_LEVEL_SHIFT_PIN);      // input
        sbi(USB_LEVEL_SHIFT_PORT, USB_LEVEL_SHIFT_PIN);     // pullup
    }
    else
    {
        sbi(USB_LEVEL_SHIFT_DDR, USB_LEVEL_SHIFT_PIN);      // input
        cbi(USB_LEVEL_SHIFT_PORT, USB_LEVEL_SHIFT_PIN);     // drive 0
    }
}



int portInit(int8_t isUSB)
{
    // initialize matrix ports - cols, rows
	// PB0-PB7 : col0 .. col7
	// PA0-PA7 : row0 .. row7
	// PC7-PC3 : row8 .. row12

    //PC 0 : reserved
    //PC 1 : reserved
    //PC 2 : reserved

    // PD0 : NUM
    // PD1 : CAPS
    // PD2 : D+ / Clock
    // PD3 : D- / Data
    // PD4 : FULL LED
    // PD5 : SCL
	// PD6 : TR pin for ZenerDiode
    // PD7 : reserved
    
	// signal direction : col -> row


	PORTA	= 0xFF;	// all rows pull-up.
	PORTB	= 0xFF;	// pull-up
	PORTC	= 0xFF;
	PORTD	= 0x00; // PD6 drive Low -5V mode, ALL_LED off
	
	DDRA	= 0x00;	// all inputs for rows
	DDRB 	= 0xFF;	// all outputs for cols
	DDRC	= 0x00;
	DDRD    = 0x7F; // PD6(zener), PD3(D+), PD2(D-), ALL_LED output

    if(isUSB)
    {
        DPpullEn(1);        // 3V Clk & Data
    }else
    {
        DPpullEn(0);        // 5V Clk & Data
    }        
    return 0;
}






int8_t checkInterface(void)
{
    uint8_t checkTimeout = 0;
    uint8_t cur_usbmode = 0;
    uint8_t prev_usbmode = 0;

    DDRD = 0; // backup register
  
    
    do{
        cbi(USBDDR, USB_CFG_DPLUS_BIT); // D+/CLK input
        sbi(USBOUT, USB_CFG_DPLUS_BIT); // D+/CLK pullUP
        
        cbi(USBDDR, USB_CFG_DMINUS_BIT); // D-/Data input
        sbi(USBOUT, USB_CFG_DMINUS_BIT); // D-/Data pullUP

        _delay_ms(10);

        if(USBIN & BV(USB_CFG_DPLUS_BIT))
        {
            cur_usbmode = 0;    // if HIGH ps2 interface
        }else{
            cur_usbmode = 1;    //if LOW  usb interface
        }
        if (prev_usbmode != cur_usbmode)
        {
            checkTimeout = INTERFACE_CHECK_TIME;    // if D+/CLK is changed start over
        }
        prev_usbmode = cur_usbmode;
    } while(checkTimeout--);

    return cur_usbmode;
}        
    



int main(void)
{

    enable_printf();
    usbmode = checkInterface();
    portInit(usbmode);

    if(usbmode)
        usbmain();

    portInit(usbmode);
    ps2main();
    return 0;
}

